from ftw.theming.utils import find_object_in_stack
from lxml import etree
from lxml.cssselect import CSSSelector
from path import Path
from scss.calculator import Calculator
from scss.extension import Extension
from scss.namespace import Namespace
from scss.types import Boolean
from scss.types import Url
from zope.dottedname.resolve import resolve
import inspect
import mimetypes


class ThemingExtensions(Extension):
    """Extensions for standard functions provided by ftw.theming.
    """

    name = 'theming'
    namespace = Namespace()


@ThemingExtensions.namespace.declare
def function_exists(name):
    calculator = inspect.currentframe().f_back.f_back.f_locals['self']
    functions = calculator.namespace._functions
    function_names = dict(functions.keys()).keys()
    return Boolean(name.render() in function_names)


@ThemingExtensions.namespace.declare
def embed_resource(relpath, fill_css=None, fill_xpath=None):
    path = find_relative_resource(relpath)
    mimetype, _compression = mimetypes.guess_type(path)
    data = path.bytes()
    data = fill_svg(data, fill_css=fill_css, fill_xpath=fill_xpath)
    return Url(u'data:{0};base64,{1}'.format(
        mimetype.decode('utf-8'),
        data.encode('base64').decode('utf-8')))


def find_relative_resource(relpath):
    calculator = find_object_in_stack('self', Calculator)
    variables = calculator.namespace.variables
    package = variables.get('$current-package').render().strip('"').strip("'")
    module = resolve(package)
    filename = variables.get('$current-relpath').render().strip('"').strip("'")
    directory = Path(module.__file__).parent.joinpath(filename).parent
    path = directory.joinpath(relpath.render().strip('"').strip("'")).abspath()
    if not path.isfile():
        raise ValueError('{0}: File does not exist: {1}'.format(
            filename, path))
    return path


def fill_svg(data, fill_css=None, fill_xpath=None):
    doc = etree.fromstring(data)
    expressions = []

    for xpr, color in fill_xpath or ():
        xpath_xpr = xpr.render().strip('"').strip("'")
        expressions.append((xpath_xpr, color))

    for xpr, color in fill_css or ():
        css_xpr = xpr.render().strip('"').strip("'")
        xpath_xpr = CSSSelector(css_xpr).path
        expressions.append((xpath_xpr, color))

    for xpath_xpr, color in expressions:
        for node in doc.xpath(xpath_xpr):
            node.attrib['fill'] = color.render()

    return etree.tostring(doc)



from collections import OrderedDict
from ftw.theming.exceptions import CyclicResourceOrder
from ftw.theming.interfaces import IDynamicSCSSResource
from ftw.theming.interfaces import ISCSSRegistry
from ftw.theming.interfaces import ISCSSResourceFactory
from ftw.theming.interfaces import SLOTS
from ftw.theming.profileinfo import ProfileInfo
from tarjan import tarjan
from zope.interface import implements


class SCSSRegistry(object):
    implements(ISCSSRegistry)

    def __init__(self):
        self.resources = []

    def add_resource(self, resource):
        self.resources.append(resource)

    def get_resources(self, context, request, profileinfo=None,
                      include_unavailable=False):
        slot_lookup = self._slot_resources_lookup(context, request)
        resources = reduce(list.__add__,
                           map(self.order_resources,
                               map(slot_lookup.get, SLOTS)))

        if not include_unavailable:
            profileinfo = profileinfo or ProfileInfo(context)
            resources = filter(
                lambda res: res.available(context, request, profileinfo),
                resources)
        return resources

    def get_raw_dynamic_resources(self, context, request):
        slot_lookup = self._slot_resources_lookup(context, request)
        resources = reduce(list.__add__, slot_lookup.values())
        return filter(IDynamicSCSSResource.providedBy, resources)

    def order_resources(self, resources):
        if not resources:
            return resources

        graph = OrderedDict((res.name, []) for res in resources)
        for res in resources:
            if res.after and res.after in graph:
                graph[res.name].append(res.after)
            if res.before and res.before in graph:
                graph[res.before].append(res.name)

        groups = tarjan(graph)

        cyclic_order = filter(lambda x: len(x) > 1, groups)
        if cyclic_order:
            raise CyclicResourceOrder(map(sorted, cyclic_order),
                                      resources, graph)

        order = reduce(list.__add__, groups)
        return sorted(resources, key=lambda res: order.index(res.name))

    def _slot_resources_lookup(self, context, request):
        result = dict((slot, []) for slot in SLOTS)
        for resource in self.resources:
            if ISCSSResourceFactory.providedBy(resource):
                resource = resource(context, request)
            if resource is not None:
                result[resource.slot].append(resource)
        return result



from datetime import datetime
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browser
from ftw.testbrowser import browsing
from ftw.testing import freeze
from ftw.theming.interfaces import ISCSSCompiler
from ftw.theming.interfaces import ISCSSRegistry
from ftw.theming.resource import DynamicSCSSResource
from ftw.theming.tests import FunctionalTestCase
from plone.app.layout.navigation.interfaces import INavigationRoot
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from zope.component import adapts
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import alsoProvides
from zope.interface import implements
from zope.interface import Interface
import re
import transaction


class TestThemingCSSView(FunctionalTestCase):

    def setUp(self):
        super(TestThemingCSSView, self).setUp()
        self.view = self.portal.restrictedTraverse('theming.css')
        self.view.invalidate_cache()
        self.portal_css = getToolByName(self.portal, 'portal_css')

    def tearDown(self):
        super(TestThemingCSSView, self).tearDown()
        self.portal_css.setDebugMode(False)

    @browsing
    def test_compiles_anonymously(self, browser):
        browser.open(view='theming.css')
        self.assertEquals('text/css; charset=utf-8',
                          browser.headers['Content-Type'])
        self.assertNotEquals('', browser.contents)

    @browsing
    def test_css_included_in_html(self, browser):
        browser.open()
        self.assert_css_url_present('http://nohost/plone/theming.css')

    @browsing
    def test_css_included_relative_to_navigation_root(self, browser):
        self.grant('Manager')
        folder = create(Builder('folder').titled('Folder'))

        browser.login().open(folder)
        self.assert_css_url_present('http://nohost/plone/theming.css')

        alsoProvides(folder, INavigationRoot)
        transaction.commit()
        browser.reload()
        self.assert_css_url_present('http://nohost/plone/folder/theming.css')

    def test_css_is_cached(self):
        self.register_compiler_mock()
        self.assertEquals(1, self.view())
        self.assertEquals(1, self.view(), 'Results should be cached but are not.')

    def test_invalidating_cache(self):
        self.register_compiler_mock()
        self.assertEquals(1, self.view())
        self.assertEquals(1, self.view(), 'Cache does not work...')
        self.view.invalidate_cache()
        self.assertEquals(2, self.view(), 'invalidate_cache() did not invalidate cache')
        self.assertEquals(2, self.view(), 'Cache does not work...')

    def test_changing_files_should_update_cache_when_debugmode_active(self):
        self.register_compiler_mock()
        self.assertEquals(1, self.view())
        self.assertEquals(1, self.view(), 'Cache does not work...')
        self.portal_css.setDebugMode(True)
        self.assertEquals(2, self.view(), 'Caching should be disabled in debug mode.')
        self.compiler().imitate_change()
        self.assertEquals(3, self.view(), 'Changing files should change cache')

    def test_recooking_portal_css_flushes_cache(self):
        self.register_compiler_mock()
        self.assertEquals(1, self.view())
        self.assertEquals(1, self.view(), 'Cache does not work...')
        self.portal_css.cookResources()
        self.assertEquals(2, self.view(), 'Recooking portal_css should flush cache.')

    @browsing
    def test_caching_active_when_debugmode_disabled(self, browser):
        self.portal_css.setDebugMode(False)
        browser.open()
        theming_css_url = self.get_css_url('http://nohost/plone/theming.css')
        self.assertIn('?cachekey=', theming_css_url, 'Missing cachekey param.')

        browser.open(theming_css_url)
        self.assertEquals('private, max-age=31536000',
                          browser.headers['Cache-Control'],
                          'Cache headers should be set.')

    @browsing
    def test_cachekey_refreshes_when_navroot_changes(self, browser):
        with freeze(datetime(2015, 1, 2, 3, 4)) as clock:
            self.portal_css.setDebugMode(False)
            self.grant('Manager')
            navroot = create(Builder('folder')
                             .titled('Folder')
                             .providing(INavigationRoot))

            browser.open(navroot)
            css_base_url = 'http://nohost/plone/folder/theming.css'
            self.assert_css_url_present(css_base_url)
            theming_css_url = self.get_css_url(css_base_url)

            clock.forward(hours=1)
            navroot.reindexObject()  # updates modified date
            transaction.commit()
            browser.reload()
            self.assertNotEqual(theming_css_url, self.get_css_url(css_base_url),
                                'Cachekey should be refreshed when navroot changes.')

    @browsing
    def test_cachekey_changes_when_dynamic_resource_cachekey_changes(self, browser):
        resource = DynamicSCSSResource('foo', cachekey='foo')
        getUtility(ISCSSRegistry).add_resource(resource)

        self.portal_css.setDebugMode(False)
        browser.open()
        theming_css_url = self.get_css_url('http://nohost/plone/theming.css')
        self.assertIn('?cachekey=', theming_css_url, 'Missing cachekey param.')

        browser.reload()
        self.assertEqual(theming_css_url,
                         self.get_css_url('http://nohost/plone/theming.css'),
                         'URL (cachekey?) did unexpectedly change.')

        resource.cachekey = 'bar'

        browser.reload()
        self.assertNotEqual(theming_css_url,
                            self.get_css_url('http://nohost/plone/theming.css'),
                            'Cachekey should have changed.')

    @browsing
    def test_requests_are_redirected_when_not_no_navroot(self, browser):
        """When configured, TinyMCE makes requests to theming.css relative to
        the current context.
        In order to reuse existing caches, we simply redirect all requests of
        theming.css on non-navroot contexts to their navroot versions, adding
        a cachkey param when necessary.
        """
        self.grant('Manager')
        self.portal_css.setDebugMode(False)
        folder = create(Builder('folder'))
        page = create(Builder('page').within(folder))

        browser.open(page, view='theming.css')
        self.assertRegexpMatches(
            browser.url,
            r'^{}'.format(
                re.escape('http://nohost/plone/theming.css?cachekey=')))

        alsoProvides(folder, INavigationRoot)
        transaction.commit()
        browser.open(page, view='theming.css')
        self.assertRegexpMatches(
            browser.url,
            r'^{}'.format(
                re.escape('http://nohost/plone/folder/theming.css?cachekey=')))

    def test_view_provides_get_url(self):
        """For internal use in templates, the view should provide the theming
        css URL.
        """
        portal_css_url = self.portal.restrictedTraverse('theming.css').get_url()
        self.assertRegexpMatches(
            portal_css_url,
            r'^{}'.format(
                re.escape('http://nohost/plone/theming.css?cachekey=')))

        self.grant('Manager')
        folder = create(Builder('folder'))
        folder_css_url = folder.restrictedTraverse('theming.css').get_url()
        self.assertEquals(portal_css_url, folder_css_url,
                          'The CSS should always be relative to the root.')

    def register_compiler_mock(self):
        sitemanager = self.portal.getSiteManager()
        COMPILER_MOCK_DATA = {'counter': 0,
                              'version': 1}

        @sitemanager.registerAdapter
        class CompilerMock(object):
            implements(ISCSSCompiler)
            adapts(IPloneSiteRoot, Interface)

            def __init__(self, context, request):
                pass

            def compile(self, debug=False):
                COMPILER_MOCK_DATA['counter'] += 1
                return COMPILER_MOCK_DATA['counter']

            def get_cachekey(self, dynamic_resources_only=False):
                return str(COMPILER_MOCK_DATA['version'])

            def imitate_change(self):
                COMPILER_MOCK_DATA['version'] += 1

    def compiler(self):
        return getMultiAdapter((self.portal, self.request), ISCSSCompiler)

    def list_css_urls(self):
        return [node.attrib.get('href') for node in browser.css('link')]

    def get_css_url(self, urlbase):
        xpr = re.compile(r'^{0}($|\?)'.format(re.escape(urlbase)))
        urls = filter(xpr.match, self.list_css_urls())
        try:
            url, = urls
        except ValueError:
            return None
        else:
            return url

    def assert_css_url_present(self, url):
        assert self.get_css_url(url), \
            'No CSS URL "{0}" found in {1}'.format(url, self.list_css_urls())



from ftw.theming.exceptions import CyclicResourceOrder
from ftw.theming.interfaces import ISCSSRegistry
from ftw.theming.interfaces import ISCSSResourceFactory
from ftw.theming.registry import SCSSRegistry
from ftw.theming.resource import DynamicSCSSResource
from ftw.theming.resource import SCSSFileResource
from ftw.theming.resource import SCSSResource
from ftw.theming.tests.stubs import CONTEXT
from ftw.theming.tests.stubs import ProfileInfoStub
from ftw.theming.tests.stubs import REQUEST
from operator import attrgetter
from unittest2 import TestCase
from zope.interface import provider
from zope.interface.verify import verifyObject


def slots(resources):
    return map(attrgetter('slot'), resources)


def paths(resources):
    return map(attrgetter('relative_path'), resources)


class TestSCSSRegistry(TestCase):

    def test_implements_interface(self):
        registry = SCSSRegistry()
        verifyObject(ISCSSRegistry, registry)

    def test_add_resources(self):
        foo = SCSSFileResource('ftw.theming.tests', 'resources/foo.scss')
        registry = SCSSRegistry()
        registry.add_resource(foo)
        self.assertEquals([foo], registry.get_resources(CONTEXT, REQUEST))

    def test_resources_are_ordered_by_slot_order(self):
        registry = SCSSRegistry()
        for slotname in ('policy', 'variables', 'addon', 'theme', 'top'):
            registry.add_resource(SCSSFileResource('ftw.theming.tests',
                                                   'resources/foo.scss',
                                                   slot=slotname))

        self.assertEquals(
            ['top', 'variables', 'addon', 'theme', 'policy'],
            slots(registry.get_resources(CONTEXT, REQUEST)))

    def test_resource_order_is_kept_within_slot(self):
        registry = SCSSRegistry()
        registry.add_resource(SCSSFileResource('ftw.theming.tests',
                                               'resources/baz.scss'))
        registry.add_resource(SCSSFileResource('ftw.theming.tests',
                                               'resources/foo.scss'))
        registry.add_resource(SCSSFileResource('ftw.theming.tests',
                                               'resources/bar.scss'))

        self.assertEquals(
            ['resources/baz.scss', 'resources/foo.scss', 'resources/bar.scss'],
            paths(registry.get_resources(CONTEXT, REQUEST)))

    def test_only_available_resources_are_returned(self):
        registry = SCSSRegistry()
        registry.add_resource(SCSSFileResource('ftw.theming.tests',
                                               'resources/foo.scss'))
        registry.add_resource(SCSSFileResource('ftw.theming.tests',
                                               'resources/bar.scss',
                                               profile='bar:default'))

        self.assertEquals(
            ['resources/foo.scss'],
            paths(registry.get_resources(CONTEXT, REQUEST, ProfileInfoStub())))

        self.assertEquals(
            ['resources/foo.scss', 'resources/bar.scss'],
            paths(registry.get_resources(CONTEXT, REQUEST,
                                         ProfileInfoStub('bar:default'))))

    def test_include_unavailable_resources(self):
        registry = SCSSRegistry()
        registry.add_resource(SCSSFileResource('ftw.theming.tests',
                                               'resources/foo.scss'))
        registry.add_resource(SCSSFileResource('ftw.theming.tests',
                                               'resources/bar.scss',
                                               profile='bar:default'))

        self.assertEquals(
            ['resources/foo.scss'],
            paths(registry.get_resources(CONTEXT, REQUEST, ProfileInfoStub())))

        self.assertEquals(
            ['resources/foo.scss', 'resources/bar.scss'],
            paths(registry.get_resources(CONTEXT, REQUEST, ProfileInfoStub(),
                                         include_unavailable=True)))

    def test_reordering_resources_within_slot_with_before(self):
        registry = SCSSRegistry()
        registry.add_resource(SCSSFileResource('ftw.theming.tests',
                                               'resources/foo.scss'))
        registry.add_resource(SCSSFileResource('ftw.theming.tests',
                                               'resources/bar.scss'))
        registry.add_resource(SCSSFileResource('ftw.theming.tests',
                                               'resources/baz.scss',
                                               before='resources/bar.scss'))

        self.assertEquals(
            ['resources/foo.scss', 'resources/baz.scss', 'resources/bar.scss'],
            paths(registry.get_resources(CONTEXT, REQUEST)))

    def test_reordering_resources_within_slot_with_after(self):
        registry = SCSSRegistry()
        registry.add_resource(SCSSFileResource('ftw.theming.tests',
                                               'resources/foo.scss',
                                               after='resources/bar.scss'))
        registry.add_resource(SCSSFileResource('ftw.theming.tests',
                                               'resources/bar.scss'))
        registry.add_resource(SCSSFileResource('ftw.theming.tests',
                                               'resources/baz.scss'))

        self.assertEquals(
            ['resources/bar.scss', 'resources/foo.scss', 'resources/baz.scss'],
            paths(registry.get_resources(CONTEXT, REQUEST)))

    def test_conflicting_order(self):
        registry = SCSSRegistry()
        registry.add_resource(SCSSFileResource('ftw.theming.tests',
                                               'resources/foo.scss',
                                               after='resources/bar.scss'))
        registry.add_resource(SCSSFileResource('ftw.theming.tests',
                                               'resources/bar.scss',
                                               after='resources/foo.scss'))
        registry.add_resource(SCSSFileResource('ftw.theming.tests',
                                               'resources/baz.scss'))

        with self.assertRaises(CyclicResourceOrder) as cm:
            registry.get_resources(CONTEXT, REQUEST)
        self.assertEquals(
            "Cyclic resource order: [['ftw.theming.tests:resources/bar.scss', "
            "'ftw.theming.tests:resources/foo.scss']]",
            str(cm.exception))

    def test_calls_registered_resource_factories(self):
        foo = SCSSFileResource('ftw.theming.tests', 'resources/foo.scss')
        dynamic = SCSSResource('dynamic-scss', slot='addon',
                               source=u'$foreground = black;')
        bar = SCSSFileResource('ftw.theming.tests', 'resources/baz.scss')

        @provider(ISCSSResourceFactory)
        def dynamic_resource_factory(context, request):
            return dynamic

        registry = SCSSRegistry()
        registry.add_resource(foo)
        registry.add_resource(dynamic_resource_factory)
        registry.add_resource(bar)

        self.assertEquals([foo, dynamic, bar],
                          registry.get_resources(CONTEXT, REQUEST))

    def test_resource_factories_returning_None_are_skipped(self):
        registry = SCSSRegistry()

        @registry.add_resource
        @provider(ISCSSResourceFactory)
        def null_resource_factory(context, request):
            return None

        self.assertEquals([], registry.get_resources(CONTEXT, REQUEST))
        self.assertEquals([], registry.get_resources(CONTEXT, REQUEST,
                                                     include_unavailable=True))

    def test_get_raw_dynamic_resources(self):
        """The get_raw_dynamic_resources method returns all dynamic resources without
        ordering or filtering. This is much more efficient and can be used for fast
        cache calculation.
        """
        foo = SCSSFileResource('ftw.theming.tests', 'resources/foo.scss')
        bar = DynamicSCSSResource('bar')

        registry = SCSSRegistry()
        registry.add_resource(foo)
        registry.add_resource(bar)

        self.assertEquals([bar],
                          registry.get_raw_dynamic_resources(None, None))



from plone.memoize import instance
from Products.CMFCore.utils import getToolByName
import re


class ProfileInfo(object):

    def __init__(self, context):
        self.context = context

    @instance.memoize
    def is_profile_installed(self, profileid):
        profileid = re.sub(r'^profile\-', '', profileid)
        quickinstaller = getToolByName(self.context, 'portal_quickinstaller')
        portal_setup = getToolByName(self.context, 'portal_setup')

        try:
            profileinfo = portal_setup.getProfileInfo(profileid)
        except KeyError:
            return False

        product = profileinfo['product']
        if quickinstaller.isProductInstallable(product) and \
                not quickinstaller.isProductInstalled(product):
            return False

        version = portal_setup.getLastVersionForProfile(profileid)
        return version != 'unknown'



from functools import partial
from plone.app.layout.navigation.interfaces import INavigationRoot
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from zope.interface import alsoProvides
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
import re


class Stub(object):

    def __init__(self, *interfaces):
        for iface in interfaces:
            alsoProvides(self, iface)


CONTEXT = Stub(IPloneSiteRoot, INavigationRoot)
REQUEST = Stub(IDefaultBrowserLayer)


class ProfileInfoStub(object):

    def __init__(self, *installed):
        self.installed = map(partial(re.sub, r'^profile\-', ''), installed)

    def is_profile_installed(self, profileid):
        profileid = re.sub(r'^profile\-', '', profileid)
        return profileid in self.installed



from ftw.theming.interfaces import ISCSSRegistry
from ftw.theming.tests import FunctionalTestCase
from zope.component import getUtility


class TestVariables(FunctionalTestCase):

    def test_portal_url_variable_resource(self):
        registry = getUtility(ISCSSRegistry)
        resources = dict((res.name, res)
                         for res in registry.get_resources(self.portal, self.request))
        self.assertIn('ftw.theming:portal_url', resources.keys())

        resource = resources['ftw.theming:portal_url']
        self.assertEquals('$portal-url: "http://nohost/plone";\n'
                          '@include declare-variables(portal-url);',
                          resource.get_source(self.portal, self.request).strip())

        self.assertEquals('variables', resource.slot)



from plone.app.layout.icons import icons
from plone.i18n.normalizer.interfaces import IIDNormalizer
from plone.memoize.instance import memoize
from zope.component import getUtility
import os.path
import re


WRAPPER_TEMPLATE = '<span class="{classes}">{content}</span>'


class CatalogBrainContentIcon(icons.CatalogBrainContentIcon):

    @memoize
    def html_tag(self):
        image_tag = super(CatalogBrainContentIcon, self).html_tag()
        if image_tag is None:
            return None
        return WRAPPER_TEMPLATE.format(classes=self.wrapper_classes(),
                                       content=image_tag)

    def wrapper_classes(self):
        normalizer = getUtility(IIDNormalizer)
        image_filename = os.path.basename(self.url)
        image_filename = re.sub(r'\.png$', r'', image_filename)
        mimetype_class = 'icon-mimetype-img-{}'.format(
            normalizer.normalize(image_filename))
        return 'mimetype-icon {}'.format(mimetype_class)



from ftw.theming.interfaces import ISCSSFileResource
from ftw.theming.resource import SCSSFileResource
from ftw.theming.tests.stubs import CONTEXT
from ftw.theming.tests.stubs import ProfileInfoStub
from ftw.theming.tests.stubs import REQUEST
from ftw.theming.tests.stubs import Stub
from path import Path
from plone.app.layout.navigation.interfaces import INavigationRoot
from unittest2 import TestCase
from zope.interface.verify import verifyObject
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


class TestSCSSFileResource(TestCase):

    def test_implements_interface(self):
        resource = SCSSFileResource('ftw.theming.tests', 'resources/foo.scss')
        verifyObject(ISCSSFileResource, resource)

    def test_create_resource(self):
        resource = SCSSFileResource('ftw.theming.tests', 'resources/foo.scss')
        self.assertTrue(resource)

    def test_requires_valid_slot(self):
        with self.assertRaises(ValueError):
            SCSSFileResource('ftw.theming.tests', 'resource/foo.scss', 'not existing slot')

    def test_package_must_exist(self):
        with self.assertRaises(ImportError):
            SCSSFileResource('some.package.that.does.not.exist', 'foo.scss')

    def test_full_path_is_generated(self):
        resource = SCSSFileResource('ftw.theming.tests', 'resources/foo.scss')
        self.assertEquals(Path(__file__).parent.joinpath('resources/foo.scss'),
                          resource.path)

    def test_file_must_exist(self):
        with self.assertRaises(OSError) as cm:
            SCSSFileResource('ftw.theming.tests', 'missing.file.scss')
        self.assertEquals(
            "[Errno 2] No such file or directory: '{0}/missing.file.scss'".format(
                Path(__file__).parent),
            str(cm.exception))

    def test_available__context_must_match_for_interface(self):
        resource = SCSSFileResource('ftw.theming.tests', 'resources/foo.scss',
                                    for_=INavigationRoot)
        self.assertTrue(resource.available(Stub(INavigationRoot), REQUEST))
        self.assertFalse(resource.available(Stub(), REQUEST))

    def test_available__request_must_match_layer_interface(self):
        resource = SCSSFileResource('ftw.theming.tests', 'resources/foo.scss',
                                    layer=IDefaultBrowserLayer)
        self.assertTrue(resource.available(CONTEXT, Stub(IDefaultBrowserLayer)))
        self.assertFalse(resource.available(CONTEXT, Stub()))

    def test_available__profile_must_be_installed_when_defined(self):
        resource = SCSSFileResource('ftw.theming.tests', 'resources/foo.scss',
                                    profile='foo.bar:default')

        self.assertTrue(resource.available(CONTEXT, REQUEST,
                                           ProfileInfoStub('foo.bar:default')))
        self.assertFalse(resource.available(CONTEXT, REQUEST, ProfileInfoStub()))

    def test_get_source(self):
        resource = SCSSFileResource('ftw.theming.tests', 'resources/foo.scss')
        self.assertMultiLineEqual(
            '\n'.join(('#foo {',
                       '  display: none;',
                       '}')),
            resource.get_source(CONTEXT, REQUEST))
        self.assertEquals(unicode, type(resource.get_source(CONTEXT, REQUEST)),
                          'Source should be unicode.')

    def test_get_cachekey_returns_a_value(self):
        # The cachekey is based on the modified date of the resource,
        # which changes and therefore we just test that it is positive.
        resource = SCSSFileResource('ftw.theming.tests', 'resources/foo.scss')
        self.assertTrue(resource.get_cachekey(None, None))



from ftw.theming.interfaces import ISCSSResourceFactory
from ftw.theming.resource import DynamicSCSSResource
from Products.CMFCore.utils import getToolByName
from zope.interface import provider
import hashlib


@provider(ISCSSResourceFactory)
def portal_url_variable_resource_factory(context, request):
    portal_url = getToolByName(context, 'portal_url')().decode('utf-8')
    cachekey = hashlib.md5(portal_url).hexdigest()
    return DynamicSCSSResource(
        'ftw.theming:portal_url', slot='variables',
        source=u'\n'.join((
                u'$portal-url: "{0}";'.format(portal_url),
                u'@include declare-variables(portal-url);')),
        cachekey=cachekey)



from setuptools import setup, find_packages
import os

version = '1.0.3'

tests_require = [
    'ftw.builder',
    'ftw.testbrowser',
    'ftw.testing',
    'plone.app.testing',
]


setup(name='ftw.theming',
      version=version,
      description="",
      long_description=open("README.rst").read() + "\n" + open(
          os.path.join("docs", "HISTORY.txt")).read(),

      classifiers=[
          "Environment :: Web Environment",
          'Framework :: Plone',
          'Framework :: Plone :: 4.3',
          "Intended Audience :: Developers",
          'License :: OSI Approved :: GNU General Public License (GPL)',
          'Programming Language :: Python',
          'Topic :: Software Development :: Libraries :: Python Modules',
      ],

      keywords='ftw theming',
      author='4teamwork AG',
      author_email='mailto:info@4teamwork.ch',
      url='https://github.com/4teamwork/ftw.theming',
      license='GPL2',

      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw'],
      include_package_data=True,
      zip_safe=False,

      install_requires=[
          'Plone',
          'cssselect',
          'ftw.upgrade',
          'lxml',
          'path.py',
          'pyScss >= 1.3',
          'six >= 1.4.0',  # six.unichr
          'setuptools',
          'tarjan',
      ],
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),

      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
)





class CyclicResourceOrder(Exception):
    """This exception is raised when the resource order is cyclic because of
    ``before`` and ``after`` statements on resources.
    The order is not reliable any more and therefore the exception is raised.
    """

    def __init__(self, cyclic_order, resources=None, graph=None):
        self.cyclic_order = cyclic_order
        self.resources = resources
        self.graph = graph
        message = 'Cyclic resource order: {0}'.format(self.cyclic_order)
        super(CyclicResourceOrder, self).__init__(message)
